@@ -17,11 +17,11 @@ def setup_seurat():
     """
     logging.info('installing packages for seurat conversion')
     logging.info('installing packages from bioconda: loomR, scater, seurat')
-    subprocess.check_output(('conda', 'install', '-y', '-c', 'bioconda', 'r-seurat==3.0.2',
-        'bioconductor-scater==1.14.0', 'r-loom==0.2.0.2'))
+    subprocess.check_output(('conda', 'install', '-y', '-c', 'bioconda', 'r-seurat>=3.0.2',
+        'bioconductor-scater>=1.14.0', 'r-loom>=0.2.0.2'))
 
     logging.info('installing anndata2ri')
-    subprocess.check_output(('pip', 'install', 'anndata2ri==1.0.2'))
+    subprocess.check_output(('pip', 'install', 'anndata2ri>=1.0.2'))
 
     logging.info('finished installation')
 
